SystemOrganization addCategory: #'ModCluster-Management-Core'!
SystemOrganization addCategory: #'ModCluster-Management-Pharo'!
SystemOrganization addCategory: #'ModCluster-Management-Test'!
SystemOrganization addCategory: #'ModCluster-Advertise-Core'!
SystemOrganization addCategory: #'ModCluster-Advertise-Pharo'!
SystemOrganization addCategory: #'ModCluster-Advertise-Test'!

GRObject subclass: #JBAdvertiseHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Advertise-Core'!

!JBAdvertiseHandler commentStamp: 'pmm 8/16/2009 11:31' prior: 0!
A JBAdvertiseHandler handles advertisements. The interface in intentially kept polymorphic to blocks.
!

!JBAdvertiseHandler methodsFor: 'public' stamp: 'pmm 9/10/2010 13:55'!
stop! !

!JBAdvertiseHandler methodsFor: 'public' stamp: 'pmm 8/16/2009 11:31'!
value: anAdvertisement
	self subclassResponsibility! !

JBAdvertiseHandler subclass: #JBAnnoucingHandler
	instanceVariableNames: 'mutex connections jvmRoute host'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Advertise-Core'!

!JBAnnoucingHandler class methodsFor: 'instance creation' stamp: 'pmm 9/7/2010 17:02'!
jvmRoute: aString
	^ self basicNew initializeWithJvmRoute: aString! !

!JBAnnoucingHandler class methodsFor: 'instance creation' stamp: 'pmm 9/7/2010 19:13'!
jvmRoute: aRouteString host: aHostString
	^ self basicNew initializeWithJvmRoute: aRouteString host: aHostString! !

!JBAnnoucingHandler methodsFor: 'private' stamp: 'pmm 9/7/2010 19:31'!
addChildrenOf: aDispatcher withPrefix: aString to: aStream
	aDispatcher defaultName isEmptyOrNil
		ifFalse: [ aStream nextPut: (JBContextAndAlias
			context: (aString, '/')
			alias: (self aliasOf: aDispatcher)) ].
	aDispatcher handlers keysAndValuesDo: [ :name :child |
		child isDispatcher
			ifFalse: [ aStream nextPut: (JBContextAndAlias
				context: (aString, '/', name)
				alias: (self aliasOf: aDispatcher)) ]
			ifTrue: [ self addChildrenOf: child withPrefix: aString, '/', name to: aStream ] ]! !

!JBAnnoucingHandler methodsFor: 'private' stamp: 'pmm 9/6/2010 16:55'!
aliasOf: aRequestHandler
	^ aRequestHandler serverHostname! !

!JBAnnoucingHandler methodsFor: 'private' stamp: 'pmm 9/7/2010 17:09'!
apatorsAndTypesDo: aTwoArgumentBlock
	WAServerManager default adaptors do: [ :each |
			| type |
			type := self typeOf: each.
			type isNil ifFalse: [
				aTwoArgumentBlock value: each value: type ] ]! !

!JBAnnoucingHandler methodsFor: 'private' stamp: 'pmm 9/7/2010 19:29'!
contextsAndAliasesOf: aRequestHandler
	^ aRequestHandler isDispatcher
		ifFalse: [
			Array with: (JBContextAndAlias
				context: '/'
				alias: (self aliasOf: aRequestHandler)) ]
		ifTrue: [
			Array streamContents: [ :stream |
				self addChildrenOf: aRequestHandler withPrefix: '' to: stream ] ]! !

!JBAnnoucingHandler methodsFor: 'private' stamp: 'pmm 9/8/2010 10:38'!
currentLoad
	^ 100! !

!JBAnnoucingHandler methodsFor: 'private' stamp: 'pmm 9/6/2010 18:44'!
entryPointsOf: aRequestHandler
	^ aRequestHandler isDispatcher
		ifFalse: [ #('/') ]
		ifTrue: [ Array streamContents: [ :stream |
			self addChildrenOf: aRequestHandler withPrefix: '' to: stream ] ]! !

!JBAnnoucingHandler methodsFor: 'initialize-release' stamp: 'pmm 9/10/2010 13:58'!
initializeWithJvmRoute: aRouteString host: aHostString
	self initialize.
	jvmRoute := aRouteString.
	host := aHostString.
	connections := GRSmallDictionary new.
	mutex := WAMutex new! !

!JBAnnoucingHandler methodsFor: 'public' stamp: 'pmm 9/10/2010 14:13'!
stop
	mutex critical: [
		connections do: [ :each |
			self apatorsAndTypesDo: [ :adaptor :type |
				| info requestHandler |
				requestHandler := adaptor requestHandler.
				info := JBNodeInformation
					host: host
					port: adaptor port
					type: type
					jvmRoute: jvmRoute
					contextsAndAliases: (self contextsAndAliasesOf: requestHandler)
					isRunning: adaptor isRunning.
				each stopAdatpor: info ].
			each stop ].
		connections := GRSmallDictionary new ]! !

!JBAnnoucingHandler methodsFor: 'private' stamp: 'pmm 9/12/2010 13:37'!
typeOf: aServerAdaptor
	aServerAdaptor class superclass name = #WAComancheRequestConverter
		ifTrue: [ ^ 'http' ].
	aServerAdaptor class name = #WASwazooAdaptor
		ifTrue: [ ^ 'http' ].
	aServerAdaptor class name = #ZnZincServerAdaptor
		ifTrue: [ ^ 'http' ].
	aServerAdaptor class name = #WAWebServerAdaptor
		ifTrue: [ ^ aServerAdaptor isSSLEnabled ifTrue: [ 'https' ] ifFalse: [ 'http' ] ].
	aServerAdaptor class superclass name = #AJPAdaptor
		ifTrue: [ ^ 'ajp' ].
	aServerAdaptor class name = #WAComancheAJPAdaptor
		ifTrue: [ ^ 'ajp' ].
	^ nil! !

!JBAnnoucingHandler methodsFor: 'public' stamp: 'pmm 9/12/2010 13:06'!
value: anAdvertisement
	| key |
	anAdvertisement hasManagerAddress
		ifFalse: [ ^self "probably shutdown or simething" ].
	key := JBManagerAddress
		host: anAdvertisement managerAddressHost
		port: anAdvertisement managerAddressPort.
	mutex critical: [
		| connection |
		connection := connections
			at: key
			ifAbsentPut: [ JBMCMConnection managerAddress: key ].
		self apatorsAndTypesDo: [ :adaptor :type |
			| info requestHandler |
			requestHandler := adaptor requestHandler.
			info := JBNodeInformation
				host: host
				port: adaptor port
				type: type
				jvmRoute: jvmRoute
				contextsAndAliases: (self contextsAndAliasesOf: requestHandler)
				isRunning: adaptor isRunning.
			connection maybeUpdateManagerWith: info ] ]! !

JBAdvertiseHandler subclass: #JBTranscriptHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Advertise-Test'!

!JBTranscriptHandler class methodsFor: 'examples' stamp: 'pmm 8/25/2009 22:57'!
example
	^JBPharoAdvertiseListener handler: self new! !

!JBTranscriptHandler methodsFor: 'public' stamp: 'pmm 8/16/2009 17:11'!
value: anAdvertisement
	Transcript show: anAdvertisement printString, String cr! !

GRObject subclass: #JBAdvertiseListener
	instanceVariableNames: 'handler parser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Advertise-Core'!

!JBAdvertiseListener commentStamp: 'pmm 8/16/2009 11:25' prior: 0!
A JBAnnounceListener is the abstract base class that listens for messages from mod_advertise.

Instance Variables
	handler:		<Object>
	parser:			<JBAdvertiseParser>

handler
	- the object that will receive the callbacks when a new advertisement has arrived
	
parser
	- parses the advertisements
!

!JBAdvertiseListener class methodsFor: 'accessing' stamp: 'pmm 8/16/2009 13:58'!
defaultPort
	"the default port to listen for advertisement"
	^23364! !

!JBAdvertiseListener methodsFor: 'private' stamp: 'pmm 8/16/2009 13:51'!
dataArrived: aString
	handler value: (parser parse: aString)! !

!JBAdvertiseListener methodsFor: 'initialize-release' stamp: 'pmm 8/16/2009 11:28'!
initializeWithHandler: aHandler
	super initialize.
	handler := aHandler.
	parser := JBAdvertiseParser new! !

!JBAdvertiseListener methodsFor: 'public' stamp: 'pmm 8/16/2009 11:26'!
start
	self subclassResponsibility! !

!JBAdvertiseListener methodsFor: 'public' stamp: 'pmm 8/16/2009 11:26'!
stop
	self subclassResponsibility! !

JBAdvertiseListener subclass: #JBPharoAdvertiseListener
	instanceVariableNames: 'port socket'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Advertise-Pharo'!

JBPharoAdvertiseListener class
	instanceVariableNames: 'instance'!

JBPharoAdvertiseListener class
	instanceVariableNames: 'instance'!

!JBPharoAdvertiseListener class methodsFor: 'instance creation' stamp: 'pmm 8/16/2009 13:58'!
handler: aHandler
	^self handler: aHandler port: self defaultPort! !

!JBPharoAdvertiseListener class methodsFor: 'instance creation' stamp: 'pmm 8/16/2009 12:26'!
handler: aHandler port: anInteger
	^self basicNew initializeWithHandler: aHandler port: anInteger! !

!JBPharoAdvertiseListener class methodsFor: 'class initialization' stamp: 'pmm 9/10/2010 20:18'!
initialize
	GRPharoPlatform current
		addToStartUpList: self;
		addToShutDownList: self! !

!JBPharoAdvertiseListener class methodsFor: 'startup' stamp: 'pmm 9/10/2010 20:16'!
shutDown
	instance isNil ifFalse: [
		instance stop ]! !

!JBPharoAdvertiseListener class methodsFor: 'public' stamp: 'pmm 9/10/2010 20:16'!
startOnHost: aHostString jvmRoute: aRouteString
	self
		startOnPort: self defaultPort
		host: aHostString
		jvmRoute: aRouteString! !

!JBPharoAdvertiseListener class methodsFor: 'public' stamp: 'pmm 9/10/2010 20:24'!
startOnLocalhostJvmRoute: aRouteString
	"self startOnLocalhostJvmRoute: 'first'"
	self
		startOnHost: '127.0.01'
		jvmRoute: aRouteString! !

!JBPharoAdvertiseListener class methodsFor: 'public' stamp: 'pmm 9/10/2010 20:14'!
startOnPort: anInteger host: aHostString jvmRoute: aRouteString
	instance isNil ifFalse: [
		instance stop ].
	instance := self
		handler: (JBAnnoucingHandler
			jvmRoute: aRouteString
			host: aHostString)
		port: anInteger.
	instance start! !

!JBPharoAdvertiseListener class methodsFor: 'startup' stamp: 'pmm 9/10/2010 20:16'!
startUp
	instance isNil ifFalse: [
		instance start ]! !

!JBPharoAdvertiseListener class methodsFor: 'class initialization' stamp: 'pmm 9/10/2010 20:18'!
unload
	GRPharoPlatform current
		removeFromStartUpList: self;
		removeFromShutDownList: self! !

!JBPharoAdvertiseListener methodsFor: 'initialize-release' stamp: 'pmm 8/16/2009 13:50'!
initializeWithHandler: aHandler port: anInteger
	super initializeWithHandler: aHandler.
	port := anInteger! !

!JBPharoAdvertiseListener methodsFor: 'private' stamp: 'pmm 8/16/2009 17:08'!
listenLoop
	| buffer s |
	buffer := String new: 65507. "max UDP packet size"
	s := socket.
	[ s isValid ] whileTrue: [
		| datagramm data read |
		datagramm := [ s receiveUDPDataInto: buffer ]
			on: ConnectionClosed
			do: [ :error | ^self ].
		read := datagramm first.
		read > 0
			ifTrue: [
				data :=  buffer copyFrom: 1 to: read.
				self dataArrived: data ]
			ifFalse: [
				(Delay forSeconds: 1) wait ] ].! !

!JBPharoAdvertiseListener methodsFor: 'public' stamp: 'pmm 9/11/2010 15:31'!
start
	socket := Socket newUDP.
	socket setPort: port.
	[ self listenLoop ]
		forkAt: Processor highIOPriority
		named: 'mod_adverise broadcast listener'! !

!JBPharoAdvertiseListener methodsFor: 'public' stamp: 'pmm 9/10/2010 13:56'!
stop
	| s |
	s := socket.
	s isNil ifFalse: [
		s destroy ].
	socket := nil.
	handler stop! !

GRObject subclass: #JBAdvertiseParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Advertise-Core'!

!JBAdvertiseParser methodsFor: 'public' stamp: 'pmm 8/16/2009 13:09'!
addLine: aString to: aDictionary
	| dotIndex |
	dotIndex := aString indexOf: $:.
	dotIndex isZero ifFalse: [
		| name value |
		name := (aString copyFrom: 1 to: dotIndex - 1) asLowercase.
		value := (aString copyFrom: dotIndex + 1 to: aString size) trimBoth.
		aDictionary at: name put: value ].! !

!JBAdvertiseParser methodsFor: 'public' stamp: 'pmm 8/16/2009 13:12'!
parse: aString
	| crlf position oldPosition headers |
	headers := Dictionary new.
	crlf := String crlf.
	position := 1.
	oldPosition := position.
	position := aString indexOfSubCollection: crlf startingAt: position.
	[ position > 0 ] whileTrue: [
		self addLine: (aString copyFrom: oldPosition to: position - 1) to: headers.
		position := position + crlf size.
		oldPosition := position.
		position := aString indexOfSubCollection: crlf startingAt: position.
	].
	^JBAdvertisement headers: headers! !

GRObject subclass: #JBAdvertisement
	instanceVariableNames: 'date sequence digest server managerAddress managerUrl managerProtocol managerHost'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Advertise-Core'!

!JBAdvertisement class methodsFor: 'instance creation' stamp: 'pmm 8/16/2009 13:19'!
headers: aDictionary
	^self basicNew
		initializeWithDate: (aDictionary at: 'date' ifAbsent: [ nil ])
		sequence: (aDictionary at: 'sequence' ifAbsent: [ nil ])
		digest: (aDictionary at: 'digest' ifAbsent: [ nil ])
		server: (aDictionary at: 'server' ifAbsent: [ nil ])
		managerAddress: (aDictionary at: 'x-manager-address' ifAbsent: [ nil ])
		managerUrl: (aDictionary at: 'x-manager-url' ifAbsent: [ nil ])
		managerProtocol: (aDictionary at: 'x-manager-protocol' ifAbsent: [ nil ])
		managerHost: (aDictionary at: 'x-manager-host' ifAbsent: [ nil ])! !

!JBAdvertisement methodsFor: 'accessing' stamp: 'pmm 8/16/2009 11:20'!
date
	^date! !

!JBAdvertisement methodsFor: 'accessing' stamp: 'pmm 8/16/2009 11:21'!
digest
	^digest! !

!JBAdvertisement methodsFor: 'testing' stamp: 'pmm 9/12/2010 13:05'!
hasManagerAddress
	^managerAddress notNil! !

!JBAdvertisement methodsFor: 'initialize-release' stamp: 'pmm 8/16/2009 13:16'!
initializeWithDate: aDateString sequence: aSequenceString digest: aDigestString server: aServerString managerAddress: aManagerAddressString managerUrl: aManagerUrlString managerProtocol: aManagerProtocolString managerHost: aManagerHostString
	super initialize.
	date := aDateString.
	sequence := aSequenceString.
	digest := aDigestString.
	server := aServerString.
	managerAddress := aManagerAddressString.
	managerUrl := aManagerUrlString.
	managerProtocol := aManagerProtocolString.
	managerHost := aManagerHostString
	! !

!JBAdvertisement methodsFor: 'accessing' stamp: 'pmm 8/16/2009 11:22'!
managerAddress
	^managerAddress! !

!JBAdvertisement methodsFor: 'accessing' stamp: 'pmm 9/6/2010 12:25'!
managerAddressHost
	^self managerAddress copyUpTo: $:! !

!JBAdvertisement methodsFor: 'accessing' stamp: 'pmm 9/6/2010 12:26'!
managerAddressPort
	^(self managerAddress copyAfter: $:) greaseInteger! !

!JBAdvertisement methodsFor: 'accessing' stamp: 'pmm 8/16/2009 11:23'!
managerHost
	^managerHost! !

!JBAdvertisement methodsFor: 'accessing' stamp: 'pmm 8/16/2009 11:22'!
managerProtocol
	^managerProtocol! !

!JBAdvertisement methodsFor: 'accessing' stamp: 'pmm 8/16/2009 11:22'!
managerUrl
	^managerUrl! !

!JBAdvertisement methodsFor: 'printing' stamp: 'pmm 8/16/2009 17:16'!
printOn: aStream
	date isNil ifFalse: [
		aStream
			nextPutAll: 'Date: ';
			nextPutAll: date;
			cr ].
	sequence isNil ifFalse: [
		aStream
			nextPutAll: 'Sequence: ';
			nextPutAll: sequence;
			cr ].
	digest isNil ifFalse: [
		aStream
			nextPutAll: 'Digest: ';
			nextPutAll: digest;
			cr ].
	server isNil ifFalse: [
		aStream
			nextPutAll: 'Server: ';
			nextPutAll: server;
			cr ].
	managerAddress isNil ifFalse: [
		aStream
			nextPutAll: 'X-Manager-Address: ';
			nextPutAll: managerAddress;
			cr ].
	managerUrl isNil ifFalse: [
		aStream
			nextPutAll: 'X-Manager-Url: ';
			nextPutAll: managerUrl;
			cr ].
	managerProtocol isNil ifFalse: [
		aStream
			nextPutAll: 'X-Manager-Protocol: ';
			nextPutAll: managerProtocol;
			cr ].
	managerHost isNil ifFalse: [
		aStream
			nextPutAll: 'X-Manager-Host: ';
			nextPutAll: managerHost;
			cr ]! !

!JBAdvertisement methodsFor: 'accessing' stamp: 'pmm 8/16/2009 11:21'!
sequence
	^sequence! !

!JBAdvertisement methodsFor: 'accessing' stamp: 'pmm 8/16/2009 11:22'!
server
	^server! !

GRObject subclass: #JBAnnouncedNode
	instanceVariableNames: 'type port'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Advertise-Core'!

!JBAnnouncedNode class methodsFor: 'instance creation' stamp: 'pmm 9/7/2010 16:45'!
type: aString port: anInteger
	^self basicNew initializeWithType: aString port: anInteger! !

!JBAnnouncedNode methodsFor: 'comparing' stamp: 'pmm 9/7/2010 16:38'!
= other
	other == self
		ifTrue: [ ^true ].
	(other isKindOf: JBAnnouncedNode)
		ifFalse: [ ^false ].
	^self type = other type
		and: [ self port = other port ]! !

!JBAnnouncedNode methodsFor: 'comparing' stamp: 'pmm 9/7/2010 16:39'!
hash
	^self type bitXor: self port hash! !

!JBAnnouncedNode methodsFor: 'initialize-release' stamp: 'pmm 9/10/2010 12:59'!
initializeWithType: aString port: anInteger
	self initialize.
	type := aString.
	port := anInteger! !

!JBAnnouncedNode methodsFor: 'accessing' stamp: 'pmm 9/10/2010 20:29'!
port
	^ port! !

!JBAnnouncedNode methodsFor: 'printing' stamp: 'pmm 9/10/2010 20:35'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: self type;
		nextPut: $:;
		print: self port;
		nextPut: $)! !

!JBAnnouncedNode methodsFor: 'accessing' stamp: 'pmm 9/10/2010 20:30'!
type
	^ type! !

GRObject subclass: #JBBalancerConf
	instanceVariableNames: 'balancer stickySession stickySessionCookie stickySessionPath stickySessionRemove stickySessionForce waitWorker maxattempts'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 22:02'!
balancer
	^ balancer! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/29/2009 10:11'!
balancer: aString
	"Name of the balancer"
	aString size > 40
		ifTrue: [ JBInvalidArgumentError signal: 'balancer must not be bigger than 40 characters' ].
	balancer := aString! !

!JBBalancerConf methodsFor: 'initialize-release' stamp: 'pmm 8/23/2009 22:13'!
initialize
	super initialize.
	self balancer: 'mycluster'.
	self stickySession: true.
	self stickySessionCookie: 'JSESSIONID'.
	self stickySessionPath: 'jsessionid'.
	self stickySessionRemove: false.
	self stickySessionForce: true.
	self waitWorker: (Duration seconds: 0).
	self maxattempts: 1! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 22:02'!
maxattempts
	^ maxattempts! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 9/5/2010 19:50'!
maxattempts: anInteger 
	maxattempts := anInteger greaseString! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 22:02'!
stickySession
	^ stickySession! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 22:05'!
stickySession: aBoolean
	"true: use JVMRoute to stick a request to a node
	false: ignore JVMRoute"
	stickySession := aBoolean ifTrue: [ 'Yes' ] ifFalse: [ 'No' ]! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 22:02'!
stickySessionCookie
	^ stickySessionCookie! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/29/2009 10:10'!
stickySessionCookie: aString
	"Name of the cookie containing the sessionid."
	aString size > 30
		ifTrue: [ JBInvalidArgumentError signal: 'StickySessionCookie must not be bigger than 30 characters' ].
	stickySessionCookie := aString! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 22:02'!
stickySessionForce
	^ stickySessionForce! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 22:06'!
stickySessionForce: aBoolean
	"true: return an error if the request can't be routed according to JVMRoute
	false: Route it to another node"
	stickySessionForce := aBoolean ifTrue: [ 'Yes' ] ifFalse: [ 'No' ]! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 22:02'!
stickySessionPath
	^ stickySessionPath! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/29/2009 10:11'!
stickySessionPath: aString
	"Name of the parameter containing the sessionid."
	aString size > 30
		ifTrue: [ JBInvalidArgumentError signal: 'StickySessionPath must not be bigger than 30 characters' ].
	stickySessionPath := aString! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 22:02'!
stickySessionRemove
	^ stickySessionRemove! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 22:10'!
stickySessionRemove: aBoolean
	"true: remove the sessionid (cookie or parameter) when the request can't be routed to the right node.
	false: send it anyway."
	stickySessionRemove := aBoolean ifTrue: [ 'Yes' ] ifFalse: [ 'No' ]! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 22:02'!
waitWorker
	^ waitWorker! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 22:12'!
waitWorker: aDuration 
	waitWorker := aDuration asSeconds! !

GRObject subclass: #JBContextAndAlias
	instanceVariableNames: 'context alias'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Advertise-Core'!

!JBContextAndAlias class methodsFor: 'instance creation' stamp: 'pmm 9/7/2010 19:28'!
context: aContextString alias: anAliasString
	^self basicNew intializeWithContext: aContextString alias: anAliasString! !

!JBContextAndAlias methodsFor: 'accessing' stamp: 'pmm 9/7/2010 19:27'!
alias
	^alias! !

!JBContextAndAlias methodsFor: 'accessing' stamp: 'pmm 9/10/2010 21:20'!
aliases
	^ alias isNil
		ifTrue: [ #() ]
		ifFalse: [ Array with: alias ]! !

!JBContextAndAlias methodsFor: 'accessing' stamp: 'pmm 9/7/2010 19:27'!
context
	^context! !

!JBContextAndAlias methodsFor: 'initialize-release' stamp: 'pmm 9/7/2010 19:28'!
intializeWithContext: aContextString alias: anAliasString
	self initialize.
	context := aContextString .
	alias := anAliasString! !

!JBContextAndAlias methodsFor: 'printing' stamp: 'pmm 9/10/2010 20:40'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: 'context: ';
		print: context;
		nextPutAll: ' alias: ';
		print: alias;
		nextPut: $)! !

GRObject subclass: #JBDocumentation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBDocumentation commentStamp: 'pmm 8/16/2009 11:15' prior: 0!
http://www.jboss.org/community/wiki/Mod-ClusterManagementProtocol
http://www.jboss.org/community/wiki/ModClusterDesign
http://www.jboss.org/community/wiki/Mod-ClusterInternals
http://www.jboss.org/mod_cluster/advertise.html

http://www.jboss.org/community/wiki/Mod-Cluster-INFO-RSP
http://www.jboss.org/community/wiki/Mod-Clusternodebalancer

http://www.jboss.org/mod_cluster/native/config.html
http://www.jboss.org/mod_cluster/java/properties.html
!

GRObject subclass: #JBMCMConnection
	instanceVariableNames: 'lastAnouncement lastAnnouncedNodes client mutex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Advertise-Core'!

!JBMCMConnection class methodsFor: 'instance creation' stamp: 'pmm 9/7/2010 16:36'!
managerAddress: aManagerAddress
	^self basicNew initializeWithAddress: aManagerAddress! !

!JBMCMConnection methodsFor: 'private' stamp: 'pmm 9/11/2010 23:41'!
configureNode: aJBNodeAnnoucementState for: aNodeInformation
	| request |
	aJBNodeAnnoucementState isConfigured
		ifTrue: [ ^self ].
	"Tomcat always seems to send an INFO request first."
	request := JBInfoRequest new.
	client sendRequest: request.
	
	request := JBConfigRequest
		jvmRoute: aNodeInformation jvmRoute
		host: aNodeInformation host
		port: aNodeInformation port
		type: aNodeInformation type.
	client sendRequest: request.
	
	aJBNodeAnnoucementState beConfigured! !

!JBMCMConnection methodsFor: 'private' stamp: 'pmm 9/11/2010 23:14'!
disableAndRemoveApplicationOf: anAnnoundedNode with: aNodeInformation
	| announcedContexts state |
	state := lastAnnouncedNodes at: anAnnoundedNode ifAbsent: [
		"if we never configured anything we don't need to remove anything"
		^ self ].
	self configureNode: state for: aNodeInformation.
	announcedContexts := state contextsAndAliases.
	self disableApplications: announcedContexts for: aNodeInformation.
	self removeApplications: announcedContexts for: aNodeInformation.
	state contextsAndAliases: #(). ! !

!JBMCMConnection methodsFor: 'private' stamp: 'pmm 9/11/2010 15:33'!
disableApplications: aCollectionOfContextAndAlias for: aNodeInformation
	self
		sendApplicationRequets: aCollectionOfContextAndAlias
		class: JBDisableApplicationRequest
		for: aNodeInformation! !

!JBMCMConnection methodsFor: 'private' stamp: 'pmm 9/10/2010 13:34'!
enableApplications: aCollectionOfContextAndAlias for: aNodeInformation
	self
		sendApplicationRequets: aCollectionOfContextAndAlias
		class: JBEnableApplicationRequest
		for: aNodeInformation! !

!JBMCMConnection methodsFor: 'initialize-release' stamp: 'pmm 9/11/2010 23:06'!
initializeWithAddress: aManagerAddress
	self initialize.
	lastAnouncement := nil.
	lastAnnouncedNodes := GRSmallDictionary new.
	mutex := WAMutex new.
	client := GRPlatform current modClusterManagementClientClass
		host: aManagerAddress host port: aManagerAddress port! !

!JBMCMConnection methodsFor: 'as yet unclassified' stamp: 'pmm 9/11/2010 23:21'!
load
	^100! !

!JBMCMConnection methodsFor: 'private' stamp: 'pmm 9/10/2010 13:44'!
maybeSendStatusUpdateFor: aNodeInformation
	
	self shouldSendStatusUpdate ifTrue: [
		self sendStatusUpdateFor: aNodeInformation.
		 lastAnouncement := DateAndTime now ]! !

!JBMCMConnection methodsFor: 'public' stamp: 'pmm 9/11/2010 23:22'!
maybeUpdateManagerWith: aNodeInformation
	| key |
	key := JBAnnouncedNode type: aNodeInformation type port: aNodeInformation port.
	mutex critical: [
		aNodeInformation isRunning
			ifTrue: [	
				| announcedContexts currentContexts newContexts removedContexts state |
				currentContexts := aNodeInformation contextsAndAliases.
				state := lastAnnouncedNodes at: key ifAbsentPut: [ JBNodeAnnoucementState newNotConfigured ].
				announcedContexts := state contextsAndAliases.
				newContexts := currentContexts copyWithoutAll: announcedContexts.
				removedContexts := announcedContexts copyWithoutAll: currentContexts.
				
				self configureNode: state for: aNodeInformation.
				self enableApplications: newContexts for: aNodeInformation.
				self disableApplications: removedContexts for: aNodeInformation.
				self removeApplications: removedContexts for: aNodeInformation.
		
				state contextsAndAliases: currentContexts.
				self maybeSendStatusUpdateFor: aNodeInformation ]
			ifFalse: [
				self disableAndRemoveApplicationOf: key with: aNodeInformation ] ]! !

!JBMCMConnection methodsFor: 'private' stamp: 'pmm 9/10/2010 21:27'!
removeApplications: aCollectionOfContextAndAlias for: aNodeInformation
	self
		sendApplicationRequets: aCollectionOfContextAndAlias
		class: JBRemoveApplicationRequest
		for: aNodeInformation! !

!JBMCMConnection methodsFor: 'private' stamp: 'pmm 9/10/2010 13:34'!
sendApplicationRequets: aCollectionOfContextAndAlias class: anApplicationClass for: aNodeInformation
	aCollectionOfContextAndAlias isEmpty
		ifTrue: [ ^self ].
	aCollectionOfContextAndAlias do: [ :each |
		| request |
		request := anApplicationClass
			jvmRoute: aNodeInformation jvmRoute
			aliases: each aliases
			context: each context.
		client sendRequest: request ]! !

!JBMCMConnection methodsFor: 'private' stamp: 'pmm 9/10/2010 13:37'!
sendStatusUpdateFor: aNodeInformation
	| request |
	request := JBStatusRequest
		jvmRoute: aNodeInformation jvmRoute
		load: self load.
	client sendRequest: request! !

!JBMCMConnection methodsFor: 'private' stamp: 'pmm 9/10/2010 13:41'!
shouldSendStatusUpdate
	^lastAnouncement isNil
		or: [ (DateAndTime now - lastAnouncement) > (Duration seconds: 9) ]! !

!JBMCMConnection methodsFor: 'public' stamp: 'pmm 9/10/2010 14:12'!
stop
	mutex critical: [
		client stop.
		lastAnnouncedNodes := GRSmallDictionary new.
		lastAnouncement := nil ]! !

!JBMCMConnection methodsFor: 'public' stamp: 'pmm 9/11/2010 22:46'!
stopAdatpor: aNodeInformation
	| key |
	key := JBAnnouncedNode type: aNodeInformation type port: aNodeInformation port.
	mutex critical: [
		self disableAndRemoveApplicationOf: key with: aNodeInformation.
		self stopNodeFor: aNodeInformation ]! !

!JBMCMConnection methodsFor: 'private' stamp: 'pmm 9/10/2010 13:35'!
stopApplications: aCollectionOfContextAndAlias for: aNodeInformation
	self
		sendApplicationRequets: aCollectionOfContextAndAlias
		class: JBStopApplicationRequest
		for: aNodeInformation! !

!JBMCMConnection methodsFor: 'private' stamp: 'pmm 9/11/2010 15:55'!
stopNodeFor: aNodeInformation
	| request |
	request := JBStopNodeRequest jvmRoute: aNodeInformation jvmRoute.
	^ client sendRequest: request! !

GRObject subclass: #JBMCMProtocolClient
	instanceVariableNames: 'host port mutex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBMCMProtocolClient class methodsFor: 'instance creation' stamp: 'pmm 9/6/2010 19:21'!
host: aString port: aNumber
	^ self basicNew initializeWithHost: aString port: aNumber! !

!JBMCMProtocolClient methodsFor: 'initialize-release' stamp: 'pmm 9/6/2010 19:21'!
initializeWithHost: aString port: aNumber
	self initialize.
	host := aString.
	port := aNumber.
	mutex := WAMutex new! !

!JBMCMProtocolClient methodsFor: 'public' stamp: 'pmm 9/6/2010 11:46'!
sendRequest: aRequest
	self subclassResponsibility! !

!JBMCMProtocolClient methodsFor: 'public' stamp: 'pmm 9/6/2010 11:47'!
stop
	self subclassResponsibility! !

JBMCMProtocolClient subclass: #JBPharoCMProtocolClient
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Pharo'!

!JBPharoCMProtocolClient methodsFor: 'private' stamp: 'pmm 9/10/2010 21:06'!
ensureValidSocket
	(stream isNil or: [ stream socket isValid not ]) ifTrue: [
		stream isNil ifFalse: [
			stream close ].
		stream := SocketStream openConnectionToHostNamed:  host port: port.
		stream timeout: Socket standardTimeout.
		stream autoFlush: false.
		stream shouldSignal: true.
		stream bufferSize: 16r2000.
		stream ascii ].! !

!JBPharoCMProtocolClient methodsFor: 'public' stamp: 'pmm 9/10/2010 20:49'!
sendRequest: aRequest
	^ mutex critical: [
		| response contentLength |

		self ensureValidSocket.
		aRequest requestStringOn: stream.
		stream flush.
		response := aRequest responseClass fromHeader: (stream upToAll: String crlf, String crlf).
		contentLength := response contentLength.
		contentLength isZero ifFalse: [
			response body: (stream next: contentLength) ].
		response ]! !

!JBPharoCMProtocolClient methodsFor: 'public' stamp: 'pmm 9/10/2010 21:04'!
stop
	mutex critical: [
		stream isNil ifTrue: [ ^self ].
		stream close.
		stream := nil ]! !

GRObject subclass: #JBManagerAddress
	instanceVariableNames: 'host port'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Advertise-Core'!

!JBManagerAddress class methodsFor: 'instance creation' stamp: 'pmm 9/6/2010 19:17'!
host: aString port: aNumber
	^ self basicNew initializeWithHost: aString port: aNumber! !

!JBManagerAddress methodsFor: 'comparing' stamp: 'pmm 9/7/2010 16:34'!
= other
	other == self
		ifTrue: [ ^true ].
	(other isKindOf: JBManagerAddress)
		ifFalse: [ ^false ].
	^self host = other host
		and: [ self port = other port ]! !

!JBManagerAddress methodsFor: 'comparing' stamp: 'pmm 9/6/2010 19:17'!
hash
	^ host hash bitXor: port hash! !

!JBManagerAddress methodsFor: 'accessing' stamp: 'pmm 9/6/2010 19:03'!
host
	^ host! !

!JBManagerAddress methodsFor: 'initialize-release' stamp: 'pmm 9/6/2010 19:04'!
initializeWithHost: aString port: aNumber
	self initialize.
	host := aString.
	port := aNumber! !

!JBManagerAddress methodsFor: 'accessing' stamp: 'pmm 9/6/2010 19:03'!
port
	^ port! !

!JBManagerAddress methodsFor: 'printing' stamp: 'pmm 9/10/2010 20:34'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: self host;
		nextPut: $:;
		print: self port;
		nextPut: $)! !

GRObject subclass: #JBNodeAnnoucementState
	instanceVariableNames: 'configured contextsAndAliases'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Advertise-Core'!

!JBNodeAnnoucementState class methodsFor: 'instance creation' stamp: 'pmm 9/11/2010 22:45'!
newConfigured
	^self new
		beConfigured;
		yourself! !

!JBNodeAnnoucementState class methodsFor: 'instance creation' stamp: 'pmm 9/11/2010 22:45'!
newNotConfigured
	^self new! !

!JBNodeAnnoucementState methodsFor: 'accessing' stamp: 'pmm 9/11/2010 22:44'!
beConfigured
	configured := true! !

!JBNodeAnnoucementState methodsFor: 'accessing' stamp: 'pmm 9/11/2010 22:42'!
contextsAndAliases
	^ contextsAndAliases! !

!JBNodeAnnoucementState methodsFor: 'accessing' stamp: 'pmm 9/11/2010 22:42'!
contextsAndAliases: anObject
	contextsAndAliases := anObject! !

!JBNodeAnnoucementState methodsFor: 'initialize-release' stamp: 'pmm 9/11/2010 23:20'!
initialize
	super initialize.
	contextsAndAliases := #().
	configured := false! !

!JBNodeAnnoucementState methodsFor: 'accessing' stamp: 'pmm 9/11/2010 22:42'!
isConfigured
	^ configured! !

GRObject subclass: #JBNodeConf
	instanceVariableNames: 'jvmRoute domain host port type flushpackets flushwait ping smax ttl timeout'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:28'!
domain
	^ domain! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/29/2009 10:11'!
domain: aString
	aString size > 20
		ifTrue: [ JBInvalidArgumentError signal: 'Domain must not be bigger than 20 characters' ].
	domain := aString! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
flushpackets
	^ flushpackets! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
flushpackets: aString
	flushpackets := aString! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:30'!
flushpacketsAuto
	"wait for flushwait time before sending"
	self flushpackets: 'Auto'! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:30'!
flushpacketsOff
	"don't flush"
	self flushpackets: 'Off'! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:30'!
flushpacketsOn
	"Send immediately"
	self flushpackets: 'On'! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
flushwait
	^ flushwait! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:36'!
flushwait: aDuration
	"Time to wait before flushing."
	flushwait := aDuration asMilliSeconds! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
host
	^ host! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
host: anObject
	host := anObject! !

!JBNodeConf methodsFor: 'initialize-release' stamp: 'pmm 8/23/2009 21:45'!
initializeWithJvmRoute: aString
	super initialize.
	"set all the defaults"
	self jvmRoute: aString.
	self domain: ''.
	self host: 'localhost'.
	self port: 8009.
	self typeAjp.
	self flushpacketsOff.
	self flushwait: (Duration milliSeconds: 10).
	self ping: (Duration seconds: 10).
	self flag: #todo. "smax"
	self ttl: (Duration seconds: 60)! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
jvmRoute
	^ jvmRoute! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/29/2009 10:11'!
jvmRoute: aString
	aString size > 80
		ifTrue: [ JBInvalidArgumentError signal: 'JvmRoute must not be bigger than 80 characters' ].
	jvmRoute := aString! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
ping
	^ ping! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:37'!
ping: aDuration
	"Time to wait for a pong answer to a ping. 0 means we don't try to ping before sending."
	ping := aDuration asSeconds! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
port
	^ port! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
port: anObject
	port := anObject! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
smax
	^ smax! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
smax: anObject
	smax := anObject! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
timeout
	^ timeout! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:39'!
timeout: aDuration
	"Max time httpd will wait for the backend connection."
	timeout := aDuration asSeconds! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
ttl
	^ ttl! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:38'!
ttl: aDuration
	"max time in seconds to life for connection above smax"
	ttl := aDuration asSeconds! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
type
	^ type! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:40'!
type: aString
	type := aString! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:40'!
typeAjp
	self type: 'ajp'! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:41'!
typeHttp
	self type: 'http'! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:41'!
typeHttps
	self type: 'https'! !

GRObject subclass: #JBNodeInformation
	instanceVariableNames: 'host port type jvmRoute contextsAndAliases isRunning'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Advertise-Core'!

!JBNodeInformation class methodsFor: 'instance creation' stamp: 'pmm 9/7/2010 19:34'!
host: aHostString port: aPortString type: aTypeString jvmRoute: aRouteString contextsAndAliases: aCollectionOfContextsAndAliases isRunning: aBoolean
	^self basicNew initializeWithHost: aHostString port: aPortString type: aTypeString jvmRoute: aRouteString contextsAndAliases: aCollectionOfContextsAndAliases isRunning: aBoolean! !

!JBNodeInformation methodsFor: 'accessing' stamp: 'pmm 9/10/2010 13:07'!
aliases
	^ contextsAndAliases collect: [ :each | each alias ]! !

!JBNodeInformation methodsFor: 'accessing' stamp: 'pmm 9/10/2010 13:06'!
contexts
	^ contextsAndAliases collect: [ :each | each context ]! !

!JBNodeInformation methodsFor: 'accessing' stamp: 'pmm 9/10/2010 13:29'!
contextsAndAliases
	^ contextsAndAliases! !

!JBNodeInformation methodsFor: 'accessing' stamp: 'pmm 9/7/2010 18:57'!
host
	^host! !

!JBNodeInformation methodsFor: 'initialize-release' stamp: 'pmm 9/7/2010 19:34'!
initializeWithHost: aHostString port: aPortString type: aTypeString jvmRoute: aRouteString contextsAndAliases: aCollectionOfContextsAndAliases isRunning: aBoolean
	self initialize.
	host := aHostString.
	port := aPortString.
	type := aTypeString.
	jvmRoute := aRouteString.
	contextsAndAliases := aCollectionOfContextsAndAliases.
	isRunning := aBoolean! !

!JBNodeInformation methodsFor: 'accessing' stamp: 'pmm 9/7/2010 18:58'!
isRunning
	^isRunning! !

!JBNodeInformation methodsFor: 'accessing' stamp: 'pmm 9/7/2010 18:58'!
jvmRoute
	^jvmRoute! !

!JBNodeInformation methodsFor: 'accessing' stamp: 'pmm 9/7/2010 18:57'!
port
	^port! !

!JBNodeInformation methodsFor: 'printing' stamp: 'pmm 9/10/2010 20:42'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: self type;
		nextPutAll: '://';
		nextPutAll: self host;
		nextPut: $:;
		print: self port;
		nextPut: $)! !

!JBNodeInformation methodsFor: 'accessing' stamp: 'pmm 9/7/2010 18:58'!
type
	^type! !

GRObject subclass: #JBRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

JBRequest subclass: #JBApplicationRequest
	instanceVariableNames: 'jvmRoute aliases context'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBApplicationRequest class methodsFor: 'instance creation' stamp: 'pmm 9/7/2010 17:12'!
jvmRoute: aJvmRouteString aliases: aCollectionOfStrings context: aString
	^self basicNew
		initializeWithJvmRoute: aJvmRouteString
		aliases: aCollectionOfStrings
		context: aString! !

!JBApplicationRequest methodsFor: 'private' stamp: 'pmm 9/7/2010 17:11'!
generateBody
	| url codec urlString bodySize |
	url := WAUrl new.
	jvmRoute isEmptyOrNil
		ifFalse: [ url addField: 'JVMRoute' value: jvmRoute ].
	aliases do: [ :each |
		url addField: 'Alias' value: each ].
	context isEmptyOrNil
		ifFalse: [ url addField: 'Context' value: context ].
	^ self justParametersOf: url! !

!JBApplicationRequest methodsFor: 'initialize-release' stamp: 'pmm 9/11/2010 15:51'!
initializeWithJvmRoute: aJvmRouteString aliases: aCollectionOfStrings context: aString
	self initialize.
	jvmRoute := aJvmRouteString.
	aliases := aCollectionOfStrings.
	context := aString! !

JBApplicationRequest subclass: #JBDisableApplicationRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBDisableApplicationRequest methodsFor: 'accessing' stamp: 'pmm 8/15/2009 18:10'!
name
	^'DISABLE-APP'! !

JBApplicationRequest subclass: #JBEnableApplicationRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBEnableApplicationRequest methodsFor: 'accessing' stamp: 'pmm 8/15/2009 18:10'!
name
	^'ENABLE-APP'! !

JBApplicationRequest subclass: #JBRemoveApplicationRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBRemoveApplicationRequest methodsFor: 'accessing' stamp: 'pmm 8/15/2009 18:11'!
name
	^'REMOVE-APP'! !

JBApplicationRequest subclass: #JBStopApplicationRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBStopApplicationRequest methodsFor: 'accessing' stamp: 'pmm 8/15/2009 18:10'!
name
	^'STOP-APP'! !

JBRequest subclass: #JBConfigRequest
	instanceVariableNames: 'jvmRoute host port type'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBConfigRequest commentStamp: 'pmm 8/15/2009 18:21' prior: 0!
Send configuration information for a node or set of nodes.
http://www.jboss.org/community/wiki/Mod-Clusternodebalancer

Instance Variables
	balanacerConf:		<Object>
	balancer:		<Object>
	configuration:		<Object>
	context:		<Object>
	domain:		<Object>
	host:		<Object>
	jvmRoute:		<Object>
	port:		<Object>
	type:		<Object>

balanacerConf
	- xxxxx

balancer
	- xxxxx

configuration
	- xxxxx

context
	- xxxxx

domain
	- xxxxx

host
	- xxxxx

jvmRoute
	- xxxxx

port
	- xxxxx

type
	- xxxxx
!

!JBConfigRequest class methodsFor: 'instance creation' stamp: 'pmm 9/11/2010 22:58'!
jvmRoute: aRouteString host: aHostString port: aPortInteger type: aTypeString
	^self basicNew
		initializeWithJvmRoute: aRouteString
		host: aHostString
		port: aPortInteger
		type: aTypeString! !

!JBConfigRequest methodsFor: 'private' stamp: 'pmm 9/11/2010 23:01'!
generateBody
	| url codec urlString bodySize |
	url := WAUrl new.
	url addField: 'JVMRoute' value: jvmRoute.
	url addField: 'Host' value: host.
	url addField: 'Port' value: port.
	url addField: 'Type' value: type.
	^ self justParametersOf: url! !

!JBConfigRequest methodsFor: 'initialization' stamp: 'pmm 9/11/2010 22:57'!
initializeWithJvmRoute: aRouteString host: aHostString port: aPortInteger type: aTypeString
	self initialize.
	jvmRoute := aRouteString.
	host := aHostString.
	port := aPortInteger.
	type := aTypeString! !

!JBConfigRequest methodsFor: 'accessing' stamp: 'pmm 9/11/2010 15:48'!
name
	^'CONFIG'! !

JBRequest subclass: #JBDumpRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBDumpRequest methodsFor: 'accessing' stamp: 'pmm 8/15/2009 18:13'!
name
	^'DUMP'! !

!JBDumpRequest methodsFor: 'accessing' stamp: 'pmm 9/7/2010 08:49'!
responseClass
	^JBDumpResponse! !

JBRequest subclass: #JBInfoRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBInfoRequest methodsFor: 'accessing' stamp: 'pmm 8/15/2009 18:13'!
name
	^'INFO'! !

!JBInfoRequest methodsFor: 'accessing' stamp: 'pmm 9/7/2010 08:49'!
responseClass
	^JBInfoResponse! !

!JBRequest methodsFor: 'private' stamp: 'pmm 9/5/2010 20:23'!
generateBody
	"Default implementation takes no arguments.
	Subclasses may want to override."
	^ ''! !

!JBRequest methodsFor: 'private' stamp: 'pmm 9/7/2010 11:41'!
justParametersOf: aUrl
	| codec urlString |
	codec := GRCodec forEncoding: 'utf-8'.
	urlString := String streamContents: [ :stream |
		| document |
		document := WAUnescapedDocument 
			on: stream
			codec: codec.
		aUrl encodeOn: document ].
	
	"urlString will start with /? skip it"
	^ urlString copyFrom: 3 to: urlString size! !

!JBRequest methodsFor: 'accessing' stamp: 'pmm 9/11/2010 15:47'!
name
	self subclassResponsibility! !

!JBRequest methodsFor: 'private' stamp: 'pmm 9/11/2010 15:44'!
requestPath
	^'/'! !

!JBRequest methodsFor: 'accessing' stamp: 'pmm 9/6/2010 11:42'!
requestString
	^String streamContents: [ :stream |
		self requestStringOn: stream ]! !

!JBRequest methodsFor: 'accessing' stamp: 'pmm 9/11/2010 23:37'!
requestStringOn: aStream
	| body |
	body := self generateBody.
	aStream
		nextPutAll: self name;
		nextPutAll: ' ';
		nextPutAll: self requestPath;
		nextPutAll: ' HTTP/1.0';
		crlf;
		nextPutAll: 'Content-Length: ';
		print: body size;
		crlf;
		nextPutAll: 'User-Agent: ClusterListener/1.0';
		crlf;
		nextPutAll: 'Connection: Keep-Alive';
		crlf;
		crlf;
		nextPutAll: body;
		crlf! !

!JBRequest methodsFor: 'accessing' stamp: 'pmm 9/6/2010 18:56'!
responseClass
	^JBGenericResponse! !

JBRequest subclass: #JBStatusRequest
	instanceVariableNames: 'jvmRoute load'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBStatusRequest commentStamp: 'pmm 8/15/2009 18:20' prior: 0!
A JBStatusRequest is xxxxxxxxx.

Instance Variables
	jvmRoute:		<String>
	load:		<String>

jvmRoute
	- is what is after the . in the JSESSIONID cookie or the parameter jsessionid. (max size: 80)

load
	- That is a number (string) It is a number between 1 and 100 and defines the normalized weighted load applied to the worker. Only possible values are real load fator. The value "0" means the node is on standby. "-1" means the node is broken (The cluster can't connect to it).
!

!JBStatusRequest class methodsFor: 'instance creation' stamp: 'pmm 9/7/2010 11:20'!
jvmRoute: aJvmRouteString load: anInteger
	^self basicNew initializeWithJvmRoute: aJvmRouteString load: anInteger! !

!JBStatusRequest methodsFor: 'private' stamp: 'pmm 9/7/2010 11:42'!
generateBody
	| url codec urlString bodySize |
	url := WAUrl new.
	jvmRoute isEmptyOrNil
		ifFalse: [ url addField: 'JVMRoute' value: jvmRoute ].
	jvmRoute isNil
		ifFalse: [ url addField:  'Load' value: load greaseString ].
	^ self justParametersOf: url! !

!JBStatusRequest methodsFor: 'initialize-release' stamp: 'pmm 9/7/2010 11:20'!
initializeWithJvmRoute: aJvmRouteString load: anInteger
	super initialize.
	jvmRoute := aJvmRouteString.
	load := anInteger! !

!JBStatusRequest methodsFor: 'accessing' stamp: 'pmm 8/15/2009 18:18'!
name
	^'STATUS'! !

!JBStatusRequest methodsFor: 'accessing' stamp: 'pmm 9/7/2010 11:01'!
responseClass
	^JBStatusResponse! !

JBRequest subclass: #JBStopNodeRequest
	instanceVariableNames: 'jvmRoute'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBStopNodeRequest class methodsFor: 'instance creation' stamp: 'pmm 9/11/2010 15:52'!
jvmRoute: aJvmRouteString
	^self basicNew initializeWithJvmRoute: aJvmRouteString! !

!JBStopNodeRequest methodsFor: 'private' stamp: 'pmm 9/11/2010 15:53'!
generateBody
	| url codec urlString bodySize |
	url := WAUrl new.
	url addField: 'JVMRoute' value: jvmRoute.
	^ self justParametersOf: url! !

!JBStopNodeRequest methodsFor: 'initialization' stamp: 'pmm 9/11/2010 15:52'!
initializeWithJvmRoute: aJvmRouteString
	super initialize.
	jvmRoute := aJvmRouteString! !

!JBStopNodeRequest methodsFor: 'accessing' stamp: 'pmm 9/11/2010 15:49'!
name
	^'REMOVE-APP'! !

!JBStopNodeRequest methodsFor: 'accessing' stamp: 'pmm 9/11/2010 15:49'!
requestPath
	^ '/*'! !

GRObject subclass: #JBResponse
	instanceVariableNames: 'protocol headers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

JBResponse subclass: #JBDumpResponse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

JBResponse subclass: #JBGenericResponse
	instanceVariableNames: 'body'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBGenericResponse methodsFor: 'accessing' stamp: 'pmm 9/7/2010 08:29'!
body
	^body! !

!JBGenericResponse methodsFor: 'accessing' stamp: 'pmm 9/7/2010 08:29'!
body: aString
	body := aString! !

JBResponse subclass: #JBInfoResponse
	instanceVariableNames: 'vhost vport rhost rport'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBInfoResponse methodsFor: 'accessing' stamp: 'pmm 9/11/2010 23:23'!
body: aString
	"not yet implemented"! !

JBResponse subclass: #JBPingResponse
	instanceVariableNames: 'id state'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBPingResponse methodsFor: 'accessing' stamp: 'pmm 9/7/2010 11:06'!
body: aString
	| parameters |
	parameters := self parametersFrom: aString.
	state := parameters at: 'State' ifAbsent: [ nil ].
	id := parameters at: 'id' ifAbsent: [ nil ]! !

!JBPingResponse methodsFor: 'accessing' stamp: 'pmm 9/7/2010 09:01'!
id
	"That is the generation id of process in httpd if it changes (increases) when httpd has been restarted and its view of the cluster
configuration could be incorrect. In this case ModClusterService should send a new CONFIG ASAP so the information could be updated."
	^ id greaseInteger! !

!JBPingResponse methodsFor: 'accessing' stamp: 'pmm 9/7/2010 08:53'!
state
	"That is the state httpd see for the node at the time of the processing (value from shared memory)."
	^ state! !

!JBResponse class methodsFor: 'instance creation' stamp: 'pmm 9/7/2010 08:29'!
fromHeader: aString
	^self basicNew initializeFromHeader: aString! !

!JBResponse methodsFor: 'private' stamp: 'pmm 9/7/2010 08:28'!
addHeaderLine: aString
	protocol isNil
		ifTrue: [ protocol := aString ]
		ifFalse: [
			| colonIndex |
			colonIndex := aString indexOf: $:.
			headers
				at: (aString copyFrom: 1 to: colonIndex - 1)
				put: (aString copyFrom: colonIndex + 2 to: aString size) ]! !

!JBResponse methodsFor: 'accessing' stamp: 'pmm 9/7/2010 08:30'!
body: aString
	self subclassResponsibility! !

!JBResponse methodsFor: 'accessing' stamp: 'pmm 9/7/2010 09:00'!
contentLength
	headers
		at: 'Content-Length'
		ifPresent: [ :value | ^ value greaseInteger ].
	^ 0! !

!JBResponse methodsFor: 'accessing' stamp: 'pmm 9/7/2010 08:28'!
headers
	^headers! !

!JBResponse methodsFor: 'initialize-release' stamp: 'pmm 9/7/2010 09:07'!
initializeFromHeader: aString
	| crlf start previous current |
	super initialize.
	crlf := String crlf.
	start := 1.
	headers := GRSmallDictionary new.
	previous := aString indexOfSubCollection: crlf startingAt: start ifAbsent: [ 0 ].
	current :=  aString indexOfSubCollection: crlf startingAt: previous + 2 ifAbsent: [ 0 ].
	[ current > (previous + 2) ] whileTrue: [
		self addHeaderLine: (aString copyFrom: start to: previous - 1).
		start := previous + 2.
		previous := current.
		current := aString indexOfSubCollection: crlf startingAt: previous + 2 ifAbsent: [ 0 ].
	].
	self addHeaderLine: (aString copyFrom: start to: previous - 1).
	self addHeaderLine: (aString copyFrom: previous + 2 to: aString size)! !

!JBResponse methodsFor: 'private' stamp: 'pmm 9/7/2010 11:05'!
parametersFrom: anUrlBody
	^(WAUrl absolute: '/?', anUrlBody) queryFields! !

!JBResponse methodsFor: 'accessing' stamp: 'pmm 9/7/2010 08:28'!
protocol
	^protocol! !

JBResponse subclass: #JBStatusResponse
	instanceVariableNames: 'jvmRoute state id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBStatusResponse methodsFor: 'accessing' stamp: 'pmm 9/7/2010 11:06'!
body: aString
	| parameters |
	parameters := self parametersFrom: aString.
	state := parameters at: 'State' ifAbsent: [ nil ].
	jvmRoute  := parameters at: 'JVMRoute' ifAbsent: [ nil ].
	id := parameters at: 'id' ifAbsent: [ nil ].! !

!JBStatusResponse methodsFor: 'accessing' stamp: 'pmm 9/7/2010 11:06'!
id
	"That is the generation id of process in httpd if it changes (increases) when httpd has been restarted and its view of the cluster
configuration could be incorrect. In this case ModClusterService should send a new CONFIG ASAP so the information could be updated."
	^ id greaseInteger! !

!JBStatusResponse methodsFor: 'accessing' stamp: 'pmm 9/7/2010 11:02'!
jvmRoute
	"is what is after the . in the JSESSIONID cookie or the parameter jsessionid. (max size: 80)"
	^ jvmRoute! !

!JBStatusResponse methodsFor: 'accessing' stamp: 'pmm 9/7/2010 11:02'!
state
	"That is the state httpd see for the node at the time of the processing (value from shared memory)."
	^ state! !

GRObject subclass: #JBTodo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBTodo commentStamp: '<historical>' prior: 0!
JBApplicationRequest >> #generateBody should probably user server codec
Might forget removed Adatpors!

!GRPharoPlatform methodsFor: '*modcluster-management-pharo' stamp: 'pmm 9/7/2010 16:47'!
modClusterManagementClientClass
	^JBPharoCMProtocolClient! !

GRError subclass: #JBError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

JBError subclass: #JBInvalidArgumentError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!GRPlatform methodsFor: '*modcluster-management-Core' stamp: 'pmm 9/7/2010 16:47'!
modClusterManagementClientClass
	self subclassResponsibility! !

TestCase subclass: #JBAdvertiseParserTest
	instanceVariableNames: 'parser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Advertise-Test'!

!JBAdvertiseParserTest methodsFor: 'private' stamp: 'pmm 9/5/2010 20:03'!
sampleMessage
	^'HTTP/1.0 200 OK', String crlf,
	'Date: Sat, 15 Aug 2009 19:15:34 GMT', String crlf,
	'Sequence: 19', String crlf,
	'Digest: 79e92a311922f736ee7c6380e06b18f7', String crlf,
	'Server: aaad5664-b31e-44f8-bab5-53c7f7ea6d62', String crlf,
	'X-Manager-Address: 192.168.1.33:6666', String crlf,
	'X-Manager-Url: /aaad5664-b31e-44f8-bab5-53c7f7ea6d62', String crlf,
	'X-Manager-Protocol: http', String crlf,
	'X-Manager-Host: 127.0.0.1', String crlf, String crlf! !

!JBAdvertiseParserTest methodsFor: 'running' stamp: 'pmm 8/16/2009 12:27'!
setUp
	parser := JBAdvertiseParser new! !

!JBAdvertiseParserTest methodsFor: 'testing' stamp: 'pmm 9/6/2010 12:26'!
testParse
	| advertisement |
	advertisement := parser parse: self sampleMessage.
	self assert: advertisement sequence = '19'.
	self assert: advertisement digest = '79e92a311922f736ee7c6380e06b18f7'.
	self assert: advertisement server = 'aaad5664-b31e-44f8-bab5-53c7f7ea6d62'.
	self assert: advertisement managerAddress = '192.168.1.33:6666'.
	self assert: advertisement managerAddressHost = '192.168.1.33'.
	self assert: advertisement managerAddressPort = 6666.
	self assert: advertisement managerUrl = '/aaad5664-b31e-44f8-bab5-53c7f7ea6d62'.
	self assert: advertisement managerProtocol = 'http'.
	self assert: advertisement managerHost = '127.0.0.1'.
	! !

TestCase subclass: #JBAnnoucingHandlerTest
	instanceVariableNames: 'handler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Advertise-Test'!

!JBAnnoucingHandlerTest methodsFor: 'running' stamp: 'pmm 9/7/2010 19:31'!
setUp
	handler := JBAnnoucingHandler jvmRoute: '33' host: 'gandalf'! !

!JBAnnoucingHandlerTest methodsFor: 'testing' stamp: 'pmm 9/7/2010 19:36'!
testEntryPointsOfDefaultName
	| rootDispachter childDispatcher entryPoints contexts |
	rootDispachter := WADispatcher new.
	rootDispachter register: WAFileHandler new at: 'zork'.
	rootDispachter defaultName: 'zork'.
	entryPoints := handler contextsAndAliasesOf: rootDispachter.
	contexts := entryPoints collect: [ :each |
		each context ].
	self assert: contexts asSet = #('/' '/zork') asSet! !

!JBAnnoucingHandlerTest methodsFor: 'testing' stamp: 'pmm 9/7/2010 19:37'!
testEntryPointsOfNestedDispatcher
	| rootDispachter childDispatcher entryPoints contexts |
	rootDispachter := WADispatcher new.
	rootDispachter register: WAFileHandler new at: 'zork'.
	childDispatcher := WADispatcher new.
	childDispatcher register: WAFileHandler new at: 'bar'.
	rootDispachter register: childDispatcher at: 'foo'.
	entryPoints := handler contextsAndAliasesOf: rootDispachter.
	contexts := entryPoints collect: [ :each |
		each context ].
	self assert: contexts asSet = #('/zork' '/foo/bar') asSet! !

!JBAnnoucingHandlerTest methodsFor: 'testing' stamp: 'pmm 9/7/2010 19:37'!
testEntryPointsOfSimpleRequestHandler
	| entryPoints contexts |
	entryPoints := handler contextsAndAliasesOf: WAFileHandler new.
	contexts := entryPoints collect: [ :each |
		each context ].
	self assert: contexts = #('/')! !

TestCase subclass: #JBConfigRequestTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Test'!

!JBConfigRequestTest methodsFor: 'testing' stamp: 'pmm 9/11/2010 23:37'!
testRequestString
	| request |
	request := JBConfigRequest
		jvmRoute: '33'
		host: '127.0.0.1'
		port: 8009
		type: 'ajp'.
	self assert: request requestString = (
		'CONFIG / HTTP/1.0', String crlf,
		'Content-Length: 45', String crlf,
		'User-Agent: ClusterListener/1.0', String crlf,
		'Connection: Keep-Alive', String crlf,
		String crlf,
		'JVMRoute=33&Host=127.0.0.1&Port=8009&Type=ajp',
		String crlf)! !

TestCase subclass: #JBContextAndAliasTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Advertise-Test'!

!JBContextAndAliasTest methodsFor: 'testing' stamp: 'pmm 9/10/2010 21:20'!
testAliases
	| contextAndAlias |
	contextAndAlias := JBContextAndAlias context: '/foo' alias: nil.
	self assert: contextAndAlias aliases = #().
	contextAndAlias := JBContextAndAlias context: '/foo' alias: 'uberhost'.
	self assert: contextAndAlias aliases = #('uberhost')! !

TestCase subclass: #JBDisableApplicationRequestTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Test'!

!JBDisableApplicationRequestTest methodsFor: 'testing' stamp: 'pmm 9/11/2010 23:37'!
testRequestString
	| request |
	request := JBDisableApplicationRequest
		jvmRoute: 'node1'
		aliases: Array new
		context: '/myapp'.
	self assert: request requestString = (
		'DISABLE-APP / HTTP/1.0', String crlf,
		'Content-Length: 31', String crlf,
		'User-Agent: ClusterListener/1.0', String crlf,
		'Connection: Keep-Alive', String crlf,
		String crlf,
		'JVMRoute=node1&Context=%2Fmyapp',
		String crlf)! !

TestCase subclass: #JBDumpRequestTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Test'!

!JBDumpRequestTest methodsFor: 'testing' stamp: 'pmm 9/11/2010 23:37'!
testRequestString
	| request |
	request := JBDumpRequest new.
	self assert: request requestString = (
		'DUMP / HTTP/1.0', String crlf,
		'Content-Length: 0', String crlf,
		'User-Agent: ClusterListener/1.0', String crlf,
		'Connection: Keep-Alive', String crlf,
		String crlf,
		'',
		String crlf)! !

TestCase subclass: #JBGenericResponseTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Test'!

!JBGenericResponseTest methodsFor: 'testing' stamp: 'pmm 9/7/2010 09:13'!
testFromHeader
	| response |
	response := JBGenericResponse fromHeader: (
		'HTTP/1.1 200 OK', String crlf,
		'Date: Sun, 05 Sep 2010 17:01:54 GMT', String crlf,
		'Server: Apache/2.2.15 (Unix) mod_cluster/1.1.x mod_ssl/2.2.15 OpenSSL/0.9.8k DAV/2 mod_jk/1.2.30', String crlf,
		'Content-Length: 0', String crlf,
		'Keep-Alive: timeout=300', String crlf,
		'Connection: Keep-Alive', String crlf,
		'Content-Type: text/plain').
	self assert: response protocol = 'HTTP/1.1 200 OK'.
	self assert: response contentLength isZero.
	self assert: response headers = (GRSmallDictionary new
		add: 'Date' -> 'Sun, 05 Sep 2010 17:01:54 GMT';
		add: 'Server' -> 'Apache/2.2.15 (Unix) mod_cluster/1.1.x mod_ssl/2.2.15 OpenSSL/0.9.8k DAV/2 mod_jk/1.2.30';
		add: 'Content-Length' -> '0';
		add: 'Keep-Alive' -> 'timeout=300';
		add: 'Connection' -> 'Keep-Alive';
		add: 'Content-Type' -> 'text/plain';
		yourself)! !

TestCase subclass: #JBInfoRequestTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Test'!

!JBInfoRequestTest methodsFor: 'testing' stamp: 'pmm 9/11/2010 23:37'!
testRequestString
	| request |
	request := JBInfoRequest new.
	self assert: request requestString = (
		'INFO / HTTP/1.0', String crlf,
		'Content-Length: 0', String crlf,
		'User-Agent: ClusterListener/1.0', String crlf,
		'Connection: Keep-Alive', String crlf,
		String crlf,
		'',
		String crlf)! !

TestCase subclass: #JBInfoResponseTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Test'!

!JBInfoResponseTest methodsFor: 'testing' stamp: 'pmm 9/7/2010 08:57'!
testFromHeader
	| response |
	response := JBInfoResponse fromHeader: (
		'HTTP/1.1 200 OK', String crlf,
		'Date: Tue, 07 Sep 2010 06:20:28 GMT', String crlf,
		'Server: Apache/2.2.15 (Unix) mod_cluster/1.1.x mod_ssl/2.2.15 OpenSSL/0.9.8k DAV/2 mod_jk/1.2.30', String crlf,
		'Content-Length: 553', String crlf,
		'Keep-Alive: timeout=300', String crlf,
		'Connection: Keep-Alive', String crlf,
		'Content-Type: text/plain').
	self assert: response protocol = 'HTTP/1.1 200 OK'.
	self assert: response contentLength = 553.
	response body:
		'Node: [1],Name: 33,Balancer: mycluster,LBGroup: ,Host: 127.0.0.1,Port: 8009,Type: ajp,Flushpackets: Off,Flushwait: 10,Ping: 10,Smax: 26,Ttl: 60,Elected: 3,Read: 22757,Transfered: 0,Connected: 0,Load: 100', String crlf,
		'Vhost: [1:1:1], Alias: localhost', String crlf,
		'Context: [1:1:1], Context: /manager, Status: ENABLED', String crlf,
		'Context: [1:1:2], Context: /, Status: ENABLED', String crlf,
		'Context: [1:1:3], Context: /load-demo, Status: ENABLED', String crlf,
		'Context: [1:1:4], Context: /docs, Status: ENABLED', String crlf,
		'Context: [1:1:5], Context: /examples, Status: ENABLED', String crlf,
		'Context: [1:1:6], Context: /host-manager, Status: ENABLED'	! !

TestCase subclass: #JBStatusRequestTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Test'!

!JBStatusRequestTest methodsFor: 'testing' stamp: 'pmm 9/11/2010 23:38'!
testRequestString
	| request |
	request := JBStatusRequest
		jvmRoute: '33'
		load: 100.
	self assert: request requestString = (
		'STATUS / HTTP/1.0', String crlf,
		'Content-Length: 20', String crlf,
		'User-Agent: ClusterListener/1.0', String crlf,
		'Connection: Keep-Alive', String crlf,
		String crlf,
		'JVMRoute=33&Load=100',
		String crlf)! !

TestCase subclass: #JBStopNodeRequestTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Test'!

!JBStopNodeRequestTest methodsFor: 'testing' stamp: 'pmm 9/11/2010 23:38'!
testRequestString
	| request |
	request := JBStopNodeRequest jvmRoute: '33'.
	self assert: request requestString = (
		'REMOVE-APP /* HTTP/1.0', String crlf,
		'Content-Length: 11', String crlf,
		'User-Agent: ClusterListener/1.0', String crlf,
		'Connection: Keep-Alive', String crlf,
		String crlf,
		'JVMRoute=33',
		String crlf)! !

TestCase subclass: #JDumpResponseTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Test'!

!JDumpResponseTest methodsFor: 'testing' stamp: 'pmm 9/7/2010 08:57'!
testFromHeader
	| response |
	response := JBDumpResponse fromHeader: (
		'HTTP/1.1 200 OK', String crlf,
		'Date: Tue, 07 Sep 2010 06:38:35 GMT', String crlf,
		'Server: Apache/2.2.15 (Unix) mod_cluster/1.1.x mod_ssl/2.2.15 OpenSSL/0.9.8k DAV/2 mod_jk/1.2.30', String crlf,
		'Content-Length: 596', String crlf,
		'Keep-Alive: timeout=300', String crlf,
		'Connection: Keep-Alive', String crlf,
		'Content-Type: text/plain').
	self assert: response protocol = 'HTTP/1.1 200 OK'.
	self assert: response contentLength = 596.
	response body:
		'balancer: [1] Name: mycluster Sticky: 1 [JSESSIONID]/[jsessionid] remove: 0 force: 1 Timeout: 0 Maxtry: 1
node: [1:1],Balancer: mycluster,JVMRoute: 33,LBGroup: [],Host: 127.0.0.1,Port: 8009,Type: ajp,flushpackets: 0,flushwait: 10,ping: 10,smax: 26,ttl: 60,timeout: 0', String crlf,
		'host: 1 [localhost] vhost: 1 node: 1', String crlf,
		'context: 1 [/host-manager] vhost: 1 node: 1 status: 1', String crlf,
		'context: 2 [/examples] vhost: 1 node: 1 status: 1', String crlf,
		'context: 3 [/docs] vhost: 1 node: 1 status: 1', String crlf,
		'context: 4 [/load-demo] vhost: 1 node: 1 status: 1', String crlf,
		'context: 5 [/] vhost: 1 node: 1 status: 1', String crlf,
		'context: 6 [/manager] vhost: 1 node: 1 status: 1'	! !

TestCase subclass: #JPingResponseTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Test'!

!JPingResponseTest methodsFor: 'testing' stamp: 'pmm 9/7/2010 08:56'!
testFromHeader
	| response |
	response := JBPingResponse fromHeader: (
		'HTTP/1.1 200 OK', String crlf,
		'Date: Tue, 07 Sep 2010 06:40:29 GMT', String crlf,
		'Server: Apache/2.2.15 (Unix) mod_cluster/1.1.x mod_ssl/2.2.15 OpenSSL/0.9.8k DAV/2 mod_jk/1.2.30', String crlf,
		'Content-Length: 37', String crlf,
		'Keep-Alive: timeout=300', String crlf,
		'Connection: Keep-Alive', String crlf,
		'Content-Type: text/plain').
	self assert: response protocol = 'HTTP/1.1 200 OK'.
	self assert: response contentLength = 37.
	response body: 'Type=PING-RSP&State=OK&id=1484878861'.
	self assert: response state = 'OK'.
	self assert: response id = 1484878861! !

TestCase subclass: #JStatusResponseTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Test'!

!JStatusResponseTest methodsFor: 'testing' stamp: 'pmm 9/7/2010 11:08'!
testFromHeader
	| response |
	response := JBStatusResponse fromHeader: (
		'HTTP/1.1 200 OK', String crlf,
		'Date: Tue, 07 Sep 2010 08:56:18 GMT', String crlf,
		'Server: Apache/2.2.15 (Unix) mod_cluster/1.1.x mod_ssl/2.2.15 OpenSSL/0.9.8k DAV/2 mod_jk/1.2.30', String crlf,
		'Content-Length: 51', String crlf,
		'Keep-Alive: timeout=300', String crlf,
		'Connection: Keep-Alive', String crlf,
		'Content-Type: text/plain').
	self assert: response protocol = 'HTTP/1.1 200 OK'.
	self assert: response contentLength = 51.
	response body: 'Type=STATUS-RSP&JVMRoute=33&State=OK&id=1484878861'.
	self assert: response state = 'OK'.
	self assert: response id = 1484878861.
	self assert: response jvmRoute = '33'! !
JBPharoAdvertiseListener initialize!
